# frozen_string_literal: true

module Deamons
  # Simple process to add wash trades to a market in order to simulate trade matching for QA and dev
  class Ngrok
    require 'utils'

    def initialize
      Process.fork do
        Process.daemon(true)
        pid = Process.pid
        redirect("#{pid}.outfile", "#{pid}.errfile")
        write_pid_file(pid, "#{pid}.pid")
        start
      end
      puts "Starting Ngrok..."
    end

    def start
      cmd = "#{ENV.fetch('NGROK')}./ngrok http 8545"
      system(cmd)
      rescue ::StandardError => e
        STDERR.puts e.backtrace
        STDERR.puts '\n'
    end

    # Attempts to write the pid of the forked process to the pid file.
    # @param [Integer]  pid
    # @param [String]  pidfile
    # @return [Integer and File]
    def write_pid_file(pid, pidfile)
      File.open pidfile, "w" do |f|
        f.write pid
      end
    rescue ::Exception => e
      $stderr.puts "While writing the PID to file, unexpected #{e.class}: #{e}"
      Process.kill "HUP", pid
    end

    # Send stdout and stderr to log files for the child process
    # @param [String]  outfile
    # @param [String]  errfile
    # @return [TrueClass]
    def redirect(outfile, errfile)
      $stdin.reopen '/dev/null'
      out = File.new outfile, "a"
      err = File.new errfile, "a"
      $stdout.reopen out
      $stderr.reopen err
      $stdout.sync = $stderr.sync = true
    end
  end
end