# frozen_string_literal: true

module Deamons
  # Simple process to add wash trades to a market in order to simulate trade matching for QA and dev
  class SimpleMm
    require 'utils'

    def initialize(api, market)
      @order_book = { bids: [], asks: [] }
      @api = api
      @market = market
      Process.fork do
        Process.daemon(true)
        pid = Process.pid
        redirect("#{pid}.outfile", "#{pid}.errfile")
        write_pid_file(pid, "#{pid}.pid")
        start
      end
      puts "SimBot started on #{market} @ freq"
    end

    def start
      count = 0
      puts "--------#{@market} -------"
      loop do
        data = Utils.binance_order_book
        populate_order_book(data)
        count += 1
      rescue ::StandardError => e
        STDERR.puts e.backtrace
        STDERR.puts '\n'
        next
      end
    end

    def populate_order_book(data)
      range = 499
      nums = (0..range).to_a.shuffle
      for i in (0..range)
        ord = data[:bids][nums[i]]
        txID = JSON.parse(@api.post_order(@market, ord[0].to_f, ord[1].to_f / 1000, 'buy'))
        puts txID
        ord.push(txID['id'])
        @api.cancel_order(@order_book[:bids][nums[i]][-1]) if @order_book[:bids].size > 400
        ord = data[:asks][nums[i]]
        txID = JSON.parse(@api.post_order(@market, ord[0].to_f, ord[1].to_f / 1000, 'sell'))
        ord.push(txID['id'])
        @api.cancel_order(@order_book[:asks][nums[i]][-1]) if @order_book[:asks].size > 400
        sleep(0.1)
      end
      @order_book = data
      puts data.inspect
    end

    # Attempts to write the pid of the forked process to the pid file.
    # @param [Integer]  pid
    # @param [String]  pidfile
    # @return [Integer and File]
    def write_pid_file(pid, pidfile)
      File.open pidfile, "w" do |f|
        f.write pid
      end
    rescue ::Exception => e
      $stderr.puts "While writing the PID to file, unexpected #{e.class}: #{e}"
      Process.kill "HUP", pid
    end

    # Send stdout and stderr to log files for the child process
    # @param [String]  outfile
    # @param [String]  errfile
    # @return [TrueClass]
    def redirect(outfile, errfile)
      $stdin.reopen '/dev/null'
      out = File.new outfile, "a"
      err = File.new errfile, "a"
      $stdout.reopen out
      $stderr.reopen err
      $stdout.sync = $stderr.sync = true
    end
  end
end