# frozen_string_literal: true

module Deamons
  # Simple process to add wash trades to a market in order to simulate trade matching for QA and dev
  class WashTrade
    require 'utils'

    def initialize(api, market, freq, vol)
      @g_count = 1
      @range = 1.015
      @api = api
      @market = market
      @freq = freq
      @volume = vol
      Process.fork do
        Process.daemon(true)
        pid = Process.pid
        redirect("#{pid}.outfile", "#{pid}.errfile")
        write_pid_file(pid, "#{pid}.pid")
        start
      end
      puts "SimBot started on #{market} @ freq #{freq}."
    end

    def start
      count = 0
      puts "--------#{@market} -------"
      loop do
        vol = rand(@volume...@volume*1.3) * (Math.sin(Time.now.hour / 6.00)
                                    + 1.00) * Utils.rand_for_hour(1.5, 2)
        price = aug_price
        @api.post_order(@market, price, vol, 'buy') if rand(1...100) < 22
        @api.post_order(@market, price, vol, 'sell') if rand(1...100) < 22
        puts "order ##{count} placed"
        delay = (1 / @freq).to_i
        sleep(rand(delay...delay + 2))
        count += 1
      rescue ::StandardError => e
        STDERR.puts e.backtrace
        next
      end
    end

    # @return [float] augmented price
    def aug_price
      if @g_count > 60
        @range = rand(1.003...1.010)
        @g_count = 0
      end
      @g_count += 1
      Utils.quote(@market) * rand(1.002...@range)
    end

    # Attempts to write the pid of the forked process to the pid file.
    # @param [Integer]  pid
    # @param [String]  pidfile
    # @return [Integer and File]
    def write_pid_file(pid, pidfile)
      File.open pidfile, "w" do |f|
        f.write pid
      end
    rescue ::Exception => e
      $stderr.puts "While writing the PID to file, unexpected #{e.class}: #{e}"
      Process.kill "HUP", pid
    end

    # Send stdout and stderr to log files for the child process
    # @param [String]  outfile
    # @param [String]  errfile
    # @return [TrueClass]
    def redirect(outfile, errfile)
      $stdin.reopen '/dev/null'
      out = File.new outfile, "a"
      err = File.new errfile, "a"
      $stdout.reopen out
      $stderr.reopen err
      $stdout.sync = $stderr.sync = true
    end
  end
end